@@ -1,7 +1,7 @@
 import os
 import numpy as np
 import matplotlib.pyplot as plt
-from multiprocessing import Pool
+from pathos.multiprocessing import ProcessingPool as Pool
 import warnings
 import copy
 import dill as pickle
@@ -191,7 +191,7 @@ class HierarchicalSwarmHandler(object):
 
         """
         self.Swarms = {self.Swarm_names[swarm_index]: Swarm(self.Hierarchical_models[0], self.NumParticlesPerSwarm,
-                                                            Omega=self.Omegas[0], PhiG= self.PhiGs[0], PhiP=self.PhiPs[0], MH_fraction=self.MH_fractions[0],
+                                                            Omega=self.Omegas[0], Phig= self.PhiGs[0], Phip=self.PhiPs[0], Mh_fraction=self.MH_fractions[0],
                                                             **self.Swarm_kwargs)
                        for swarm_index in self.Swarm_names}
 
@@ -426,7 +426,7 @@ class HierarchicalSwarmHandler(object):
         num_particles = positions.shape[0]
 
         newswarm = Swarm(self.Hierarchical_models[self.Hierarchical_model_counter+ 1],num_particles,
-                         Omega=Omega, PhiP=PhiP, PhiG=PhiG, MH_fraction=MH_fraction , **self.Swarm_kwargs)
+                         Omega=Omega, Phip=PhiP, Phig=PhiG, Mh_fraction=MH_fraction , **self.Swarm_kwargs)
 
         newswarm.EvolutionCounter = 0
         newswarm.Points = positions
@@ -450,6 +450,7 @@ class HierarchicalSwarmHandler(object):
 
             newswarm.Velocities = (ptp_vel_bounds) * np.random.random_sample(size=(num_particles,self.Ndim)) - ptp_vel_bounds/2
 
+
         newswarm.BestKnownPoints = copy.deepcopy(newswarm.Points)
 
         # Recalculate best personal known values:
@@ -462,9 +463,305 @@ class HierarchicalSwarmHandler(object):
         newswarm.BestKnownSwarmPoint = newswarm.BestKnownPoints[np.argmax(newswarm.BestKnownValues)]
         newswarm.BestKnownSwarmValue = np.max(newswarm.BestKnownValues)
 
+        # Sets up multiprocessing pool for parallel function computations
+        newswarm.Pool = Pool(self.Swarm_kwargs['Nthreads'])
+
         return (newswarm)
 
+    import numpy as np
 
+    import matplotlib.pyplot as plt
+
+    from matplotlib.pyplot import pause, draw
+
+    from matplotlib import colors
+
+    from scipy.optimize import curve_fit
+
+     
+
+    fig = plt.figure()
+
+    ax = fig.gca()
+
+     
+
+    size = 50
+
+    board = np.round_(np.random.uniform(size=(size, size)))
+
+    # board = np.ones((size, size))
+
+    colors_list = ['black', 'green', 'red']
+
+    cmap = colors.ListedColormap(colors_list)
+
+    board_image = ax.imshow(board, cmap=cmap)
+
+    # setting the probabilities
+
+    p = 0.01
+
+    f = 0.001
+
+    repeat = 20
+
+    # only show inner boarders for grid
+
+     
+
+    list_nr_trees = []
+
+    list_nr_empty = []
+
+    all_areas = []
+
+     
+
+    for _ in range(repeat):
+
+       
+
+        new = board
+
+    nr_trees = 0
+
+    nr_empty = 0
+
+    # print('initial board: ', board)
+
+ 
+
+for j in range(size - 1):
+
+    for i in range(size - 1):
+
+           
+
+              # check f# check if empty, prob. grow tree
+
+if board[i, j] == 0 and np.random.rand() <= p:
+    new[i, j] = 1
+
+             
+
+              # if cell has tree, prob. fire
+
+if board[i, j] == 1 and np.random.rand() <= f:
+    new[i, j] = 2  # or fire and spread
+
+                
+
+              # if fire, spread it
+
+if board[i, j] == 2:
+    new[i, j] = 0
+
+               
+
+                
+
+                  # get rid of all of these and try and set a condition
+
+# if fire at the edges, set to 0 and leave it there.
+
+                
+
+                if i - 1 <= 0:
+
+    new[i - 1, j] = 0
+
+elif j - 1 <= 0:
+
+    new[i, j - 1] = 0
+
+elif i + 1 >= size:
+
+    new[i + 1, j] = 0
+
+elif j + 1 >= size:
+
+    new[i, j + 1] = 0
+
+               
+
+                 else:
+
+                      # sometimes the fire jumps and there are too many lit up ==> check the else conditions
+
+if new[i - 1, j] == 1:
+    new[i - 1, j] = 2
+
+if new[i, j - 1] == 1:
+    new[i, j - 1] = 2
+
+if new[i + 1, j] == 1:
+    new[i + 1, j] = 2
+
+if new[i, j + 1] == 1:
+    new[i, j + 1] = 2
+
+ 
+
+# trying to keep track of the number of trees vs empty spaces
+
+nr_trees = nr_trees + (new[i, j] == 1)
+
+nr_empty = nr_empty + (new[i, j] == 0)
+
+ 
+
+# making the graph to see how nr of empty spaces and trees will balance out
+
+    list_nr_trees = np.append(list_nr_trees, nr_trees)
+
+list_nr_empty = np.append(list_nr_empty, nr_empty)
+
+   
+
+      # Kopelman Algorithm
+
+    largest_label = 0
+
+label = np.zeros((size, size))
+
+   
+
+    for i in range(size):  # row
+
+    for j in range(size):  # column
+
+        if board[i, j] == 1:
+
+            left = label[i, j - 1]
+
+            above = label[i - 1, j]
+
+            if left == 0 and above == 0:
+                largest_label = largest_label + 1
+
+                label[i, j] = largest_label
+
+   
+
+                 elif left != 0 and above == 0:
+
+label[i, j] = left
+
+   
+
+                 elif left == 0 and above != 0:
+
+label[i, j] = above
+
+   
+
+                 else:
+
+# if both above and left are occupied, take up the label from above
+
+label[i, j] = above
+
+# let's give the coordinates of left then, to change in the 'label' matrix
+
+label[i, j - 1] = above
+
+# change all labels from the left to equal inital one from above
+
+label = np.where(label == left, above, label)
+
+'''for x in range(size):
+
+    for y in range(size):
+
+        label[x, y] = np.where(label[x, y] == left, above, label[x, y])'''
+
+   
+
+      # print('after kopelman: ', label)
+
+   
+
+      # counting the areas of each cluster
+
+areas = []
+
+for n in range(size):
+
+    a = 0 
+
+            for i in range(size):
+
+        for j in range(size):  # iterating over each element in the matrix
+
+            if label[i, j] == (n + 1):
+                a = a + 1
+
+    areas = np.append(areas, a)
+
+    areas = np.delete(areas, np.where(areas == 0))
+
+       
+
+        all_areas = np.append(all_areas, areas)  # keeping track of areas even through multiple repeats
+
+           
+
+     
+
+    # substitute the board with the new matrix
+
+    board = new 
+
+        
+
+        
+
+        image = ax.imshow(board, cmap=cmap)
+
+    pause(0.1)
+
+       
+
+        
+
+    # print(all_areas)
+
+     
+
+    def power_law(x, a, b):
+
+        return a * x ** b
+
+     
+
+    x = np.arange(1, np.max(all_areas), 1)
+
+    # y = plt.hist(all_areas, bins = x)
+
+     
+
+    popt, pcov = curve_fit(power_law, x[:-1], y[0])
+
+    # plt.plot(x, power_law(x, *popt))
+
+       
+
+    # plotting the nr of trees / empty cells
+
+    '''
+
+x = np.arange(0, repeat)
+
+plt.figure()
+
+plt.plot(x, list_nr_trees)
+
+plt.plot(x, list_nr_empty)
+
+plt.show()
+
+'''
 
 
 
@@ -613,6 +910,9 @@ class HierarchicalSwarmHandler(object):
                 if self.Hierarchical_model_counter+1 == len(self.Hierarchical_models):
                     print('\n All swarms stalled on the last model, finishing up!')
                     self.swarm_stepping_done = True
+                    for swarm in self.Swarms:
+                        swarm.Pool.close()
+                        swarm.Pool.join()
 
                 else:
                     print('\n All swarms stalled! Switching segments from ', str(self.Hierarchical_models[self.Hierarchical_model_counter].segment_number),
